/*
 * Copyright (c) 2001-2004 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: scatter_inter.c,v 1.1 2004/03/09 20:07:49 adharurk Exp $
 *
 * Program to test Intercommunicator Scatter
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <mpi.h> 
#include <lamtest_error.h>

#define MAXLEN 10000

static void intercomm_scatter(MPI_Comm inter, int rank, int crank);

int main(int argc, char **argv ) 
{ 
  int size, rank, port_length, color, local_rank;
  MPI_Comm comm, intercomm;
  char *port_name;
  int buf = 1;

  MPI_Init(&argc, &argv);

  lamtest_need_even(__FILE__, __LINE__);
  lamtest_check_size(__FILE__, __LINE__, 2, 1);

  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  /* Even rank gets 1 while odd ranks get 0 value for color */
  color = (rank % 2) ? 0 : 1;
  MPI_Comm_split(MPI_COMM_WORLD, color, rank, &comm);
  
  if (color) {
    /* even ranks */
    MPI_Comm_rank (comm, &local_rank);
    if (local_rank == 0) {
      port_name = malloc(MPI_MAX_PORT_NAME * sizeof(char));
      MPI_Open_port(MPI_INFO_NULL, port_name);
      port_length = strlen(port_name) + 1;
      
      MPI_Send(&port_length, 1, MPI_INT, rank + 1, 1, MPI_COMM_WORLD);
      MPI_Send(port_name, port_length, MPI_CHAR, rank + 1, 1, MPI_COMM_WORLD);
    }

    MPI_Comm_accept(port_name, MPI_INFO_NULL, 0, comm, &intercomm);
    intercomm_scatter(intercomm, local_rank, -1);
    MPI_Comm_disconnect(&intercomm);

    if (local_rank == 0) {
      MPI_Close_port(port_name);
      free(port_name);
    }

  } else {
    /* odd ranks */
    MPI_Comm_rank (comm, &local_rank);
    if (local_rank == 0) {
      MPI_Recv(&port_length, 1, MPI_INT, rank - 1, 1, MPI_COMM_WORLD, 
	       MPI_STATUS_IGNORE);
      
      /* allocate memory for the the port name */
      port_name = malloc(port_length * sizeof(char));
      MPI_Recv(port_name, port_length, MPI_CHAR, rank - 1, 1, MPI_COMM_WORLD, 
	       MPI_STATUS_IGNORE);
    }

    /* connect to the server */
    MPI_Comm_connect(port_name, MPI_INFO_NULL, 0, comm, &intercomm);
    intercomm_scatter(intercomm, -1, local_rank);
    MPI_Comm_disconnect(&intercomm);
    
    if (local_rank == 0) 
      free(port_name);
  }
  MPI_Comm_free(&comm);
  MPI_Finalize();
  return 0;
} 

static void
intercomm_scatter(MPI_Comm inter, int rank, int crank)
{
  int root,*out,*in,i,j,k;
  int tasks;

  if (rank != -1)
    MPI_Comm_size(inter, &tasks);
  else
    MPI_Comm_remote_size(inter, &tasks);
  in = malloc(sizeof(int) * MAXLEN);
  out = malloc(sizeof(int) * MAXLEN * tasks);
  if (in == NULL || out == NULL)
    lamtest_error(__FILE__, __LINE__, "Doh!  Rank %d was not able to allocate enough memory.  MPI test aborted!\n", rank);
  
  for(j=1,root=0;j<=MAXLEN;j*=10,root=(root+1)%tasks)  {
    if(rank == root) {
      for(i=0;i<j*tasks;i++)  out[i] = i;
      MPI_Scatter(out,j,MPI_INT,in,j,MPI_INT,MPI_ROOT,inter);
    } else if (rank == -1) {
      MPI_Scatter(out,j,MPI_INT,in,j,MPI_INT,root,inter);
      for(k=0;k<j;k++) {
	if(in[k] != k+crank*j) {  
	  lamtest_error(__FILE__, __LINE__, "task %d: bad answer (%d) at index %d of %d (should be %d)\n",crank,in[k],k,j,k+crank*j); 
	  break; 
	}
      }
    } else {
      MPI_Scatter(out,j,MPI_INT,in,j,MPI_INT,MPI_PROC_NULL,inter);
    }
  }
  free(in);
  free(out);
}
